@@ -1,18 +1,28 @@
-/*  Arg_parser - A simple argument parser written in C++
-    Copyright (C) 2006, 2007 Antonio Diaz Diaz.
+/*  Arg_parser - POSIX/GNU command line argument parser. (C++ version)
+    Copyright (C) 2006-2015 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
+    This library is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
+    This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
+
+    As a special exception, you may use this file as part of a free
+    software library without restriction.  Specifically, if other files
+    instantiate templates or use macros or inline functions from this
+    file, or you compile this file and link it with other files to
+    produce an executable, this file does not by itself cause the
+    resulting executable to be covered by the GNU General Public
+    License.  This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General
+    Public License.
 */
 
 #include <cstring>
@@ -23,9 +33,9 @@
 
 
 bool Arg_parser::parse_long_option( const char * const opt, const char * const arg,
-                                    const Option options[], int & argind ) throw()
+                                    const Option options[], int & argind )
   {
-  unsigned int len;
+  unsigned len;
   int index = -1;
   bool exact = false, ambig = false;
 
@@ -33,7 +43,7 @@ bool Arg_parser::parse_long_option( const char * const opt, const char * const a
 
   // Test all long options for either exact match or abbreviated matches.
   for( int i = 0; options[i].code != 0; ++i )
-    if( options[i].name && !std::strncmp( options[i].name, &opt[2], len ) )
+    if( options[i].name && std::strncmp( options[i].name, &opt[2], len ) == 0 )
       {
       if( std::strlen( options[i].name ) == len )	// Exact match found
         { index = i; exact = true; break; }
@@ -45,25 +55,31 @@ bool Arg_parser::parse_long_option( const char * const opt, const char * const a
 
   if( ambig && !exact )
     {
-    _error = "option `"; _error += opt; _error += "' is ambiguous";
+    error_ = "option '"; error_ += opt; error_ += "' is ambiguous";
     return false;
     }
 
   if( index < 0 )		// nothing found
     {
-    _error = "unrecognized option `"; _error += opt; _error += '\'';
+    error_ = "unrecognized option '"; error_ += opt; error_ += '\'';
     return false;
     }
 
   ++argind;
   data.push_back( Record( options[index].code ) );
 
-  if( opt[len+2] )		// `--<long_option>=<argument>' syntax
+  if( opt[len+2] )		// '--<long_option>=<argument>' syntax
     {
     if( options[index].has_arg == no )
       {
-      _error = "option `--"; _error += options[index].name;
-      _error += "' doesn't allow an argument";
+      error_ = "option '--"; error_ += options[index].name;
+      error_ += "' doesn't allow an argument";
+      return false;
+      }
+    if( options[index].has_arg == yes && !opt[len+3] )
+      {
+      error_ = "option '--"; error_ += options[index].name;
+      error_ += "' requires an argument";
       return false;
       }
     data.back().argument = &opt[len+3];
@@ -72,10 +88,10 @@ bool Arg_parser::parse_long_option( const char * const opt, const char * const a
 
   if( options[index].has_arg == yes )
     {
-    if( !arg )
+    if( !arg || !arg[0] )
       {
-      _error = "option `--"; _error += options[index].name;
-      _error += "' requires an argument";
+      error_ = "option '--"; error_ += options[index].name;
+      error_ += "' requires an argument";
       return false;
       }
     ++argind; data.back().argument = arg;
@@ -87,27 +103,27 @@ bool Arg_parser::parse_long_option( const char * const opt, const char * const a
 
 
 bool Arg_parser::parse_short_option( const char * const opt, const char * const arg,
-                                     const Option options[], int & argind ) throw()
+                                     const Option options[], int & argind )
   {
   int cind = 1;			// character index in opt
 
   while( cind > 0 )
     {
     int index = -1;
-    const unsigned char code = opt[cind];
+    const unsigned char c = opt[cind];
 
-    if( code != 0 )
+    if( c != 0 )
       for( int i = 0; options[i].code; ++i )
-        if( code == options[i].code )
+        if( c == options[i].code )
           { index = i; break; }
 
     if( index < 0 )
       {
-      _error = "invalid option -- "; _error += code;
+      error_ = "invalid option -- '"; error_ += c; error_ += '\'';
       return false;
       }
 
-    data.push_back( Record( code ) );
+    data.push_back( Record( c ) );
     if( opt[++cind] == 0 ) { ++argind; cind = 0; }	// opt finished
 
     if( options[index].has_arg != no && cind > 0 && opt[cind] )
@@ -118,7 +134,8 @@ bool Arg_parser::parse_short_option( const char * const opt, const char * const
       {
       if( !arg || !arg[0] )
         {
-        _error = "option requires an argument -- "; _error += code;
+        error_ = "option requires an argument -- '"; error_ += c;
+        error_ += '\'';
         return false;
         }
       data.back().argument = arg; ++argind; cind = 0;
@@ -129,7 +146,7 @@ bool Arg_parser::parse_short_option( const char * const opt, const char * const
 
 
 Arg_parser::Arg_parser( const int argc, const char * const argv[],
-                        const Option options[], const bool in_order ) throw()
+                        const Option options[], const bool in_order )
   {
   if( argc < 2 || !argv || !options ) return;
 
@@ -139,12 +156,12 @@ Arg_parser::Arg_parser( const int argc, const char * const argv[],
   while( argind < argc )
     {
     const unsigned char ch1 = argv[argind][0];
-    const unsigned char ch2 = ( ch1 ? argv[argind][1] : 0 );
+    const unsigned char ch2 = ch1 ? argv[argind][1] : 0;
 
     if( ch1 == '-' && ch2 )		// we found an option
       {
       const char * const opt = argv[argind];
-      const char * const arg = (argind + 1 < argc) ? argv[argind+1] : 0;
+      const char * const arg = ( argind + 1 < argc ) ? argv[argind+1] : 0;
       if( ch2 == '-' )
         {
         if( !argv[argind][2] ) { ++argind; break; }	// we found "--"
@@ -158,10 +175,10 @@ Arg_parser::Arg_parser( const int argc, const char * const argv[],
       else { data.push_back( Record() ); data.back().argument = argv[argind++]; }
       }
     }
-  if( _error.size() ) data.clear();
+  if( error_.size() ) data.clear();
   else
     {
-    for( unsigned int i = 0; i < non_options.size(); ++i )
+    for( unsigned i = 0; i < non_options.size(); ++i )
       { data.push_back( Record() ); data.back().argument.swap( non_options[i] ); }
     while( argind < argc )
       { data.push_back( Record() ); data.back().argument = argv[argind++]; }
@@ -170,7 +187,7 @@ Arg_parser::Arg_parser( const int argc, const char * const argv[],
 
 
 Arg_parser::Arg_parser( const char * const opt, const char * const arg,
-                        const Option options[] ) throw()
+                        const Option options[] )
   {
   if( !opt || !opt[0] || !options ) return;
 
@@ -181,7 +198,7 @@ Arg_parser::Arg_parser( const char * const opt, const char * const arg,
       { if( opt[2] ) parse_long_option( opt, arg, options, argind ); }
     else
       parse_short_option( opt, arg, options, argind );
-    if( _error.size() ) data.clear();
+    if( error_.size() ) data.clear();
     }
   else { data.push_back( Record() ); data.back().argument = opt; }
   }
