@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2003-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -17,10 +17,13 @@
 
 #include <algorithm>
 #include <cstdio>
+#include <cstdlib>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
 #include "rectangle.h"
+#include "segment.h"
 #include "ucs.h"
 #include "bitmap.h"
 #include "blob.h"
@@ -31,32 +34,25 @@
 
 // First attempt at recognition without relying on context.
 //
-void Character::recognize1( const Charset & charset, const Rectangle & charbox ) throw()
+void Character::recognize1( const Charset & charset, const Rectangle & charbox )
   {
-  if( blobs() == 1 ) recognize11( charset, charbox );
+  if( blobs() == 1 )
+    {
+    const Blob & b = blob( 0 );
+    if( b.holes() == 0 ) recognize110( charset, charbox );
+    else if( b.holes() == 1 ) recognize111( charset, charbox );
+    else if( b.holes() == 2 ) recognize112( charbox );
+    }
   else if( blobs() == 2 ) recognize12( charset, charbox );
   else if( blobs() == 3 ) recognize13( charset, charbox );
   }
 
 
-// Recognizes 1 blob characters.
-// 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghklmnopqrstuvwxyz
-// #$&'()*+,-./<>@[\]^_`{|}~
-//
-void Character::recognize11( const Charset & charset, const Rectangle & charbox ) throw()
-  {
-  const Blob & b = blob( 0 );
-  if( b.holes() == 0 ) recognize110( charset, charbox );
-  else if( b.holes() == 1 ) recognize111( charset, charbox );
-  else if( b.holes() == 2 ) recognize112( charbox );
-  }
-
-
 // Recognizes 1 blob characters without holes.
 // 12357CEFGHIJKLMNSTUVWXYZcfhklmnrstuvwxyz
 // '()*+,-./<>@[\]^_`{|}~
 //
-void Character::recognize110( const Charset & charset, const Rectangle & charbox ) throw()
+void Character::recognize110( const Charset & charset, const Rectangle & charbox )
   {
   const Blob & b = blob( 0 );
   Features f( b );
@@ -70,13 +66,14 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
   if( b.height() < 5 || ( b.height() < 8 && b.width() < 6 ) ||
       b.height() > 10 * b.width() || 5 * b.height() < b.width() ) return;
 
-  code = f.test_CEFIJLlT( charset ); if( code ) { add_guess( code, 0 ); return; }
-  code = f.test_frst( charbox );     if( code ) { add_guess( code, 0 ); return; }
-  code = f.test_G();                 if( code ) { add_guess( code, 0 ); return; }
-  code = f.test_c();                 if( code ) { add_guess( code, 0 ); return; }
+  code = f.test_EFIJLlT( charset, charbox );
+  if( code ) { add_guess( code, 0 ); return; }
+  code = f.test_frst( charbox );   if( code ) { add_guess( code, 0 ); return; }
+  code = f.test_G();               if( code ) { add_guess( code, 0 ); return; }
+  code = f.test_c();               if( code ) { add_guess( code, 0 ); return; }
   if( charset.enabled( Charset::iso_8859_9 ) )
-    { code = f.test_s_cedilla();     if( code ) { add_guess( code, 0 ); return; } }
-  code = f.test_235Esz( charset );   if( code ) { add_guess( code, 0 ); return; }
+    { code = f.test_s_cedilla();   if( code ) { add_guess( code, 0 ); return; } }
+  code = f.test_235Esz( charset ); if( code ) { add_guess( code, 0 ); return; }
   code = f.test_HKMNUuvwYy( charbox );
   if( code == 'u' && f.lp.istpit() )	// Looks for merged 'tr'
     {
@@ -86,23 +83,17 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
       col = b.seek_left( b.vcenter(), col - 1, false );
       while( --col > b.hpos( 40 ) &&
              ( b.seek_top( b.vcenter(), col ) > b.top() ||
-               f.hp[col-b.left()] > b.height() / 10 ) );
-      if( col > b.hpos( 40 ) && col < b.right() )
-        {
-        only_guess( 0, b.left() ); add_guess( 't', col );
-        add_guess( 'r', b.right() ); return;
-        }
+               f.hp[col-b.left()] > b.height() / 10 ) ) ;
+      if( col > b.hpos( 40 ) && col < b.right() &&
+          set_merged_guess( 't', col, 'r', 0 ) ) return;
       }
     }
   if( code == 'N' && b.width() > b.height() && b.top() >= charbox.top() &&
       4 * f.tp[f.tp.pos(50)] < b.height() )
     {					// Looks for merged 'rv'
     const int col = f.hp.iminimum();
-    if( col > f.hp.pos( 40 ) && col < f.hp.pos( 50 ) )
-      {
-      only_guess( 0, b.left() ); add_guess ( 'r', b.left() + col );
-      add_guess( 'v', b.right() ); return;
-      }
+    if( col >= f.hp.pos( 40 ) && col < f.hp.pos( 50 ) &&
+        set_merged_guess( 'r', b.left() + col, 'v', 0 ) ) return;
     }
   if( code ) { add_guess( code, 0 ); return; }
   const int noise = ( std::min( b.height(), b.width() ) / 30 ) + 1;
@@ -111,7 +102,7 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
         ( b.height() >= 16 && f.bp.minima( b.height() / 8 ) == 2 ) ) )
     {
     code = f.test_hknwx( charbox );
-    if( code == 'n' )	// Looks for '"' or merged 'rr' or 'TT' or 'rt' or 'fl'
+    if( code == 'n' )	// Looks for '"' or merged 'rt' or 'fl'
       {
       if( b.bottom() <= charbox.vcenter() ) { add_guess( '"', 0 ); return; }
       if( b.width() > b.height() && 10 * f.lp[f.lp.pos(10)] < b.width() &&
@@ -119,25 +110,13 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
         {
         const int rgap = f.rp[f.rp.pos(50)];
         if( 10 * rgap > b.width() && !b.escape_top( b.vcenter(), b.right() ) )
-          {
-          only_guess( 0, b.left() );
-          const int lgap = f.lp[f.lp.pos(50)];
-          if( 10 * lgap > b.width() && !b.escape_top( b.vcenter(), b.left() ) &&
-              Ocrad::similar( lgap, rgap, 10 ) && b.top() < charbox.top() )
-            { add_guess( 'T', b.hcenter() ); add_guess( 'T', b.right() ); }
-          else { add_guess( 'r', b.hcenter() ); add_guess( 'r', b.right() ); }
-          return;
-          }
+          return;	// leave 'rr', 'TT', 'rz', 'FT' etc, for next pass
         }
       if( 2 * f.lp[f.lp.pos(10)] > b.width() && !f.rp.increasing( f.rp.pos( 75 ) ) )
         {
         const int col = b.seek_left( b.vcenter(), b.right() );
-        if( col <= b.hpos( 95 ) && !b.escape_top( b.vcenter(), col ) )
-          {
-          only_guess( 0, b.left() );
-          add_guess( 'r', b.hcenter() ); add_guess( 't', b.right() );
-          return;
-          }
+        if( col <= b.hpos( 95 ) && !b.escape_top( b.vcenter(), col ) &&
+            set_merged_guess( 'r', b.hcenter(), 't', 0 ) ) return;
         }
       if( f.rp.minima() == 1 && !f.rp.increasing( f.rp.pos( 75 ) ) )
         {
@@ -152,12 +131,8 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
             int col, limit = b.seek_right( b.vcenter(), b.hcenter() );
             for( col = b.hcenter(); col <= limit; ++col )
               if( b.seek_bottom( b.vcenter(), col ) < b.bottom() ) break;
-            if( col > b.left() && col < b.right() )
-              {
-              only_guess( 0, b.left() );
-              add_guess( 'f', col - 1 ); add_guess( 'l', b.right() );
-              return;
-              }
+            if( col > b.left() && col < b.right() &&
+                set_merged_guess( 'f', col - 1, 'l', 0 ) ) return;
             }
           }
         }
@@ -174,24 +149,18 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
             if( f.hp[i-b.left()] < hmin )
               { hmin = f.hp[i-b.left()]; col = i; }
           if( col > b.left() && col < b.right() )
-            { only_guess( 0, b.left() ); add_guess( 'r', col - 1 );
-            add_guess( 'f', b.right() ); }
+            set_merged_guess( 'r', col - 1, 'f', 0 );
           }
         return;
         }
       if( f.rp.isctip( 30 ) )
-        {
-        only_guess( 0, b.left() ); add_guess( 'f', b.hcenter() );
-        add_guess( 'i', b.right() ); return;
-        }
+        { set_merged_guess( 'f', b.hcenter(), 'i', 0 ); return; }
       }
     else if( code == 'k' )	// Looks for merged 'rt'
       {
-      if( 2 * f.lp[f.lp.pos(10)] > b.width() && !f.rp.increasing( f.rp.pos( 75 ) ) )
-        {
-        only_guess( 0, b.left() ); add_guess( 'r', b.hcenter() );
-        add_guess( 't', b.right() ); return;
-        }
+      if( 2 * f.lp[f.lp.pos(10)] > b.width() &&
+          !f.rp.increasing( f.rp.pos( 75 ) ) &&
+          set_merged_guess( 'r', b.hcenter(), 't', 0 ) ) return;
       }
     if( code ) { add_guess( code, 0 ); return; }
     }
@@ -201,15 +170,16 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
         f.lp.minima() == 2 && f.rp.minima() == 2 )
       { add_guess( '*', 0 ); return; }
     if( b.id( b.vcenter(), b.hcenter() ) == 0 &&
+        b.id( b.vcenter() - 1, b.hcenter() ) == 0 &&
+        b.id( b.vcenter() + 1, b.hcenter() ) == 0 &&
         b.seek_left( b.vcenter(), b.hcenter() ) <= b.hpos( 25 ) )
       {					// Found merged 'rn'
       int row = b.vpos( 95 );
       int col = b.seek_right( row, b.left() );
       col = b.seek_right( row, col + 1, false );
       col = b.seek_right( row, col + 1 );
-      if( col > b.left() && col < b.right() )
-        { only_guess( 0, b.left() ); add_guess( 'r', col );
-        add_guess( 'n', b.right() ); return; }
+      if( col > b.left() && col < b.right() &&
+          set_merged_guess( 'r', col, 'n', 0 ) ) return;
       }
     if( f.tp.minima( b.height() / 3 ) == 1 ) add_guess( 'm', 0 );
     return;
@@ -220,16 +190,18 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
     int col = b.seek_right( row, b.left() );
     col = b.seek_right( row, col + 1, false );
     col = b.seek_right( row, col + 1 );
-    if( col > b.left() && col < b.right() )
-      { only_guess( 0, b.left() ); add_guess( 'r', col );
-      add_guess( 'm', b.right() ); return; }
+    if( col > b.left() && col < b.right() &&
+        set_merged_guess( 'r', col, 'm', 0 ) ) return;
     }
 
   if( f.tp.minima( b.height() / 4 ) == 3 )
     {
+    if( f.segments_in_row( b.vcenter() ) == 2 &&
+        f.segments_in_row( b.vpos( 80 ) ) == 3 ) return;	// merged 'IX'
     int hdiff;
     if( !b.bottom_hook( &hdiff ) &&
-        ( f.vscan()[b.vcenter() - b.top()] < 4 || !b.escape_top( b.vcenter(), b.hcenter() ) ) )
+        ( f.segments_in_row( b.vcenter() ) < 4 ||
+          !b.escape_top( b.vcenter(), b.hcenter() ) ) )
       add_guess( 'w', 0 );
     return;
     }
@@ -245,7 +217,7 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
 // Recognizes 1 blob characters with 1 hole.
 // 0469ADOPQRabdegopq#
 //
-void Character::recognize111( const Charset & charset, const Rectangle & charbox ) throw()
+void Character::recognize111( const Charset & charset, const Rectangle & charbox )
   {
   const Blob & b = blob( 0 );
   const Bitmap & h = b.hole( 0 );
@@ -286,7 +258,7 @@ void Character::recognize111( const Charset & charset, const Rectangle & charbox
         Blob & b1 = const_cast< Blob & >( b );
         Blob b2( b );
         b1.bottom( row ); b2.top( row + 1 );
-        bpv.push_back( new Blob( b2 ) );
+        blobpv.push_back( new Blob( b2 ) );
         }
       }
     }
@@ -296,7 +268,7 @@ void Character::recognize111( const Charset & charset, const Rectangle & charbox
 // Recognizes 1 blob characters with 2 holes.
 // 8BQg$&
 //
-void Character::recognize112( const Rectangle & charbox ) throw()
+void Character::recognize112( const Rectangle & charbox )
   {
   const Blob & b = blob( 0 );
   const Bitmap & h1 = b.hole( 0 );		// upper hole
@@ -346,8 +318,9 @@ void Character::recognize112( const Rectangle & charbox ) throw()
       if( b.bottom_hook( &hdiff ) && hdiff > b.height() / 2 )
         if( b.top_hook( &hdiff ) && hdiff > b.height() / 2 )
           { add_guess( 's', 0 ); return; }
-    
-      if( lp.isflats() && !lp.ispit() ) { add_guess( 'B', 0 ); return; }
+
+      if( lp.isflats() && ( lp.istip() || ( lp.isflat() && b.test_BD() ) ) )
+        { add_guess( 'B', 0 ); return; }
 
       int col1 = h1.seek_left( h1.bottom(), h1.right() + 1 ) - 1;
       int col2 = h2.seek_right( h2.top(), h2.left() - 1 ) + 1;
