@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2003-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -16,9 +16,13 @@
 */
 
 #include <cstdio>
+#include <cstdlib>
 #include <vector>
+#include <stdint.h>
+
 #include "common.h"
 #include "rectangle.h"
+#include "segment.h"
 #include "ucs.h"
 #include "bitmap.h"
 #include "blob.h"
@@ -30,7 +34,7 @@
 // Recognizes 2 blob characters.
 // ij!%:;=?|
 //
-void Character::recognize12( const Charset & charset, const Rectangle & charbox ) throw()
+void Character::recognize12( const Charset & charset, const Rectangle & charbox )
   {
   const Blob & b1 = blob( 0 );		// upper blob
   const Blob & b2 = blob( 1 );		// lower blob
@@ -71,20 +75,18 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
         { add_guess( UCS::PLUSMIN, 0 ); return; }
       }
     if( b1.includes_hcenter( b2 ) && b2.includes_hcenter( b1 ) )
-      { if( b1.holes() && b2.holes() ) { add_guess( 'g', 0 ); return; } }
-    if( b1.hcenter() < b2.hcenter() )
       {
-      // Looks for merged 'fi'
+      if( b1.holes() && b2.holes() ) { add_guess( 'g', 0 ); return; }
+      }
+    if( b1.hcenter() < b2.hcenter() )		// Looks for merged 'fi'
+      {
       if( b2.height() > b2.width() && b1.hcenter() < b2.left() &&
           b1.includes_hcenter( b2 ) && 4 * b1.height() > 5 * b2.height() &&
           Ocrad::similar( b1.bottom()-b1.top(), b2.bottom()-b1.top(), 10 ) )
         {
         Character c2( new Blob( b2 ) ); c2.recognize1( charset, charbox );
-        if( c2.maybe('l') || c2.maybe('|') )
-          {
-          only_guess( 0, b1.left() ); add_guess( 'f', b2.left() );
-          add_guess( 'i', b2.right() ); return;
-          }
+        if( ( c2.maybe('l') || c2.maybe('|') ) &&
+            set_merged_guess( 'f', b2.left() - 1, 'i', 0 ) ) return;
         }
       }
     }
@@ -101,7 +103,7 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
       int code = f1.test_solid( charbox );		//FIXME all this
       if( code == '-' && 2 * b1.height() > b1.width() ) code = '.';
       else if( code == '\'' || code == '|' ) code = '.';
-      if( !code && !b1.holes() && 
+      if( !code && !b1.holes() &&
           2 * b1.height() < b2.height() && b1.width() <= b2.width() )
         {
         if( 10 * a1 >= 7 * b1.height() * b1.width() ) code = '.';
@@ -123,14 +125,12 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
               if( b2.seek_bottom( b2.vcenter(), col ) < b2.bottom() ) break;
             if( b2.left() < col && col < b2.right() )
               {
-              only_guess( 0, b2.left() );
               if( charset.enabled( Charset::iso_8859_9 ) && f2.rp.istip() )
-                { add_guess( 'T', col - 1 ); add_guess( UCS::CIDOT, b2.right() ); }
+                set_merged_guess( 'T', col - 1, UCS::CIDOT, 1 );
               else
                 {
-                add_guess( 'r', col - 1 );
-                if( code == '.' ) add_guess( 'i', b2.right() );
-                else add_guess( UCS::SIACUTE, b2.right() );
+                const int code2 = ( ( code == '.' ) ? 'i' : (int)UCS::SIACUTE );
+                set_merged_guess( 'r', col - 1, code2, 1 );
                 }
               return;
               }
@@ -166,7 +166,7 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
                   charset.enabled( Charset::iso_8859_9 ) )
                 if( !f2.lp.isctip() && f2.wp.max() >= f1.wp.max() &&
                     ( 3 * f2.wp[f2.wp.pos(10)] < 2 * f1.wp.max() ||
-                      b1.left() <= b2.left() && b2.vpos( 80 ) > charbox.bottom() ) )
+                      ( b1.left() <= b2.left() && b2.vpos( 80 ) > charbox.bottom() ) ) )
                   { add_guess( UCS::IEXCLAM, 0 ); return; }
               if( maybe_j ) add_guess( 'j', 0 ); else add_guess( 'i', 0 );
               return;
@@ -186,14 +186,24 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
         }
       }
 
-    // upper blob has no holes
-    if( ( b1.bottom() < b2.vcenter() || 2 * a1 < a2 ) && !b1.holes() )
+    if( ( !b1.holes() && ( b1.bottom() < b2.vcenter() || 2 * a1 < a2 ) ) ||
+        ( b1.holes() == 1 && b1.bottom() < b2.top() &&
+          b2.top() - b1.bottom() < b1.height() ) )
       {
       Character c( new Blob( b2 ) ); c.recognize1( charset, charbox );
       if( c.guesses() )
         {
         int code = c.guess( 0 ).code;
-        if( b1.bottom() < b2.vcenter() )
+        if( b1.holes() == 1 )
+          {
+          if( code == 'a' ) code = UCS::SARING;
+          else if( code == 'A' ) code = UCS::CARING;
+          else code = 0;
+          }
+        else if( code == 'u' &&
+                 5 * b1.width() <= b2.width() && 5 * b1.height() <= b2.width() )
+          return;
+        else if( b1.bottom() < b2.vcenter() )
           {
           int atype = '\'';
           if( UCS::isvowel( code ) && 2 * b1.width() > 3 * b1.height() &&
@@ -203,11 +213,11 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
                    ( f1.rp.decreasing() || f1.tp.increasing() ) &&
                    ( f1.bp.decreasing() || f1.lp.increasing() ) ) atype = '`';
           code = UCS::compose( code, atype );
-          if( charset.only( Charset::ascii ) )
-            {
-            if( UCS::base_letter( code ) == 'i' ) code = 'i';
-            else code = c.guess( 0 ).code;
-            }
+          }
+        if( code != c.guess( 0 ).code && charset.only( Charset::ascii ) )
+          {
+          if( UCS::base_letter( code ) == 'i' ) code = 'i';
+          else code = c.guess( 0 ).code;
           }
         if( code ) add_guess( code, 0 );
         }
@@ -218,12 +228,13 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
   if( b1.bottom() <= b2.top() )
     {
     const int code = f2.test_solid( charbox );
-    if( !b1.holes() && ( code == '.' || 
+    if( !b1.holes() && ( code == '.' ||
           ( code && Ocrad::similar( b2.height(), b2.width(), 50 ) ) ) )
       {
       if( Ocrad::similar( b1.width(), b2.width(), 50 ) && !f1.lp.isctip() )
         { add_guess( '!', 0 ); return; }
-      add_guess( '?', 0 ); return;
+      if( f1.bp.minima() == 1 ) add_guess( '?', 0 );
+      return;
       }
     if( code == '-' || code == '_' )
       if( charset.enabled( Charset::iso_8859_15 ) ||
@@ -232,9 +243,12 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
         if( b1.holes() == 1 )
           {
           const Bitmap & h = b1.hole( 0 );
-          if( Ocrad::similar( h.left() - b1.left(), b1.right() - h.right(), 40 ) )
-            { add_guess( UCS::MASCORD, 0 ); return; }
-          add_guess( UCS::FEMIORD, 0 ); return;
+          if( b2.width() >= h.width() && b2.top() - b1.bottom() < h.height() )
+            {
+            if( Ocrad::similar( h.left() - b1.left(), b1.right() - h.right(), 40 ) )
+              { add_guess( UCS::MASCORD, 0 ); return; }
+            add_guess( UCS::FEMIORD, 0 ); return;
+            }
           }
         }
     }
