@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2003-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -17,7 +17,9 @@
 
 #include <algorithm>
 #include <cstdio>
+#include <cstdlib>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
 #include "rectangle.h"
@@ -26,101 +28,99 @@
 #include "profile.h"
 
 
-Profile::Profile( const Bitmap & b, Type t ) throw()
-  : _bitmap( &b ), type( t ),
-    _limit( -1 ), _max( -1 ), _min( -1 ), _mean( -1 ),
-    _isconcave( -1 ), _isconvex( -1 ), _isflat( -1 ), _isflats( -1 ),
-    _ispit( -1 ), _istpit( -1 ), _isupit( -1 ), _isvpit( -1 ), _istip( -1 ) {}
+Profile::Profile( const Bitmap & bm_, const Type t )
+  : bm( bm_ ), type( t ),
+    limit_( -1 ), max_( -1 ), min_( -1 ), mean_( -1 ),
+    isconcave_( -1 ), isconvex_( -1 ), isflat_( -1 ), isflats_( -1 ),
+    ispit_( -1 ), istpit_( -1 ), isupit_( -1 ), isvpit_( -1 ), istip_( -1 ) {}
 
 
-void Profile::initialize() throw()
+void Profile::initialize()
   {
-  const Bitmap & b = *_bitmap;
-
   switch( type )
     {
     case left :
-      data.resize( b.height() ); _limit = b.width();
-      for( int row = b.top(); row <= b.bottom(); ++row )
+      data.resize( bm.height() ); limit_ = bm.width();
+      for( int row = bm.top(); row <= bm.bottom(); ++row )
         {
-        int j = b.left();
-        while( j <= b.right() && !b.get_bit( row, j ) ) ++j;
-        data[row-b.top()] = j - b.left();
+        int j = bm.left();
+        while( j <= bm.right() && !bm.get_bit( row, j ) ) ++j;
+        data[row-bm.top()] = j - bm.left();
         } break;
     case top :
-      data.resize( b.width() ); _limit = b.height();
-      for( int col = b.left(); col <= b.right(); ++col )
+      data.resize( bm.width() ); limit_ = bm.height();
+      for( int col = bm.left(); col <= bm.right(); ++col )
         {
-        int j = b.top();
-        while( j <= b.bottom() && !b.get_bit( j, col ) ) ++j;
-        data[col-b.left()] = j - b.top();
+        int j = bm.top();
+        while( j <= bm.bottom() && !bm.get_bit( j, col ) ) ++j;
+        data[col-bm.left()] = j - bm.top();
         } break;
     case right :
-      data.resize( b.height() ); _limit = b.width();
-      for( int row = b.top(); row <= b.bottom(); ++row )
+      data.resize( bm.height() ); limit_ = bm.width();
+      for( int row = bm.top(); row <= bm.bottom(); ++row )
         {
-        int j = b.right();
-        while( j >= b.left() && !b.get_bit( row, j ) ) --j;
-        data[row-b.top()] = b.right() - j;
+        int j = bm.right();
+        while( j >= bm.left() && !bm.get_bit( row, j ) ) --j;
+        data[row-bm.top()] = bm.right() - j;
         } break;
     case bottom :
-      data.resize( b.width() ); _limit = b.height();
-      for( int col = b.left(); col <= b.right(); ++col )
+      data.resize( bm.width() ); limit_ = bm.height();
+      for( int col = bm.left(); col <= bm.right(); ++col )
         {
-        int j = b.bottom();
-        while( j >= b.top() && !b.get_bit( j, col ) ) --j;
-        data[col-b.left()] = b.bottom() - j;
+        int j = bm.bottom();
+        while( j >= bm.top() && !bm.get_bit( j, col ) ) --j;
+        data[col-bm.left()] = bm.bottom() - j;
         } break;
     case height :
-      data.resize( b.width() ); _limit = b.height();
-      for( int col = b.left(); col <= b.right(); ++col )
+      data.resize( bm.width() ); limit_ = bm.height();
+      for( int col = bm.left(); col <= bm.right(); ++col )
         {
-        int u = b.top(), d = b.bottom();
-        while( u <= d && !b.get_bit( u, col ) ) ++u;
-        while( u <= d && !b.get_bit( d, col ) ) --d;
-        data[col-b.left()] = d - u + 1;
+        int u = bm.top(), d = bm.bottom();
+        while( u <= d && !bm.get_bit( u, col ) ) ++u;
+        while( u <= d && !bm.get_bit( d, col ) ) --d;
+        data[col-bm.left()] = d - u + 1;
         } break;
     case width :
-      data.resize( b.height() ); _limit = b.width();
-      for( int row = b.top(); row <= b.bottom(); ++row )
+      data.resize( bm.height() ); limit_ = bm.width();
+      for( int row = bm.top(); row <= bm.bottom(); ++row )
         {
-        int l = b.left(), r = b.right();
-        while( l <= r && !b.get_bit( row, l ) ) ++l;
-        while( l <= r && !b.get_bit( row, r ) ) --r;
-        data[row-b.top()] = r - l + 1;
+        int l = bm.left(), r = bm.right();
+        while( l <= r && !bm.get_bit( row, l ) ) ++l;
+        while( l <= r && !bm.get_bit( row, r ) ) --r;
+        data[row-bm.top()] = r - l + 1;
         } break;
     }
   }
 
 
-int Profile::mean() throw()
+int Profile::mean()
   {
-  if( _mean < 0 )
+  if( mean_ < 0 )
     {
-    if( _limit < 0 ) initialize();
-    _mean = 0;
-    for( int i = 0; i < samples(); ++i ) _mean += data[i];
-    if( samples() > 1 ) _mean /= samples();
+    if( limit_ < 0 ) initialize();
+    mean_ = 0;
+    for( int i = 0; i < samples(); ++i ) mean_ += data[i];
+    if( samples() > 1 ) mean_ /= samples();
     }
-  return _mean;
+  return mean_;
   }
 
 
-int Profile::max() throw()
+int Profile::max()
   {
-  if( _max < 0 )
+  if( max_ < 0 )
     {
-    if( _limit < 0 ) initialize();
-    _max = data[0];
-    for( int i = 1; i < samples(); ++i ) if( data[i] > _max ) _max = data[i];
+    if( limit_ < 0 ) initialize();
+    max_ = data[0];
+    for( int i = 1; i < samples(); ++i ) if( data[i] > max_ ) max_ = data[i];
     }
-  return _max;
+  return max_;
   }
 
 
-int Profile::max( int l, int r ) throw()
+int Profile::max( const int l, int r )
   {
-  if( _limit < 0 ) initialize();
+  if( limit_ < 0 ) initialize();
   if( r < 0 ) r = samples() - 1;
   int m = 0;
   for( int i = l; i <= r; ++i ) if( data[i] > m ) m = data[i];
@@ -128,75 +128,76 @@ int Profile::max( int l, int r ) throw()
   }
 
 
-int Profile::min() throw()
+int Profile::min()
   {
-  if( _min < 0 )
+  if( min_ < 0 )
     {
-    if( _limit < 0 ) initialize();
-    _min = data[0];
-    for( int i = 1; i < samples(); ++i ) if( data[i] < _min ) _min = data[i];
+    if( limit_ < 0 ) initialize();
+    min_ = data[0];
+    for( int i = 1; i < samples(); ++i ) if( data[i] < min_ ) min_ = data[i];
     }
-  return _min;
+  return min_;
   }
 
 
-int Profile::min( int l, int r ) throw()
+int Profile::min( const int l, int r )
   {
-  if( _limit < 0 ) initialize();
+  if( limit_ < 0 ) initialize();
   if( r < 0 ) r = samples() - 1;
-  int m = _limit;
+  int m = limit_;
   for( int i = l; i <= r; ++i ) if( data[i] < m ) m = data[i];
   return m;
   }
 
 
-int Profile::operator[]( int i ) throw()
+int Profile::operator[]( int i )
   {
-  if( _limit < 0 ) initialize();
+  if( limit_ < 0 ) initialize();
   if( i < 0 ) i = 0;
   else if( i >= samples() ) i = samples() - 1;
   return data[i];
   }
 
 
-int Profile::area( int l, int r ) throw()
+int Profile::area( const int l, int r )
   {
-  if( _limit < 0 ) initialize();
-  if( r < 0 ) r = samples() - 1;
-  int area = 0;
-  for( int i = l; i <= r; ++i ) area += data[i];
-  return area;
+  if( limit_ < 0 ) initialize();
+  if( r < 0 || r >= samples() ) r = samples() - 1;
+  int a = 0;
+  for( int i = l; i <= r; ++i ) a += data[i];
+  return a;
   }
 
 
-bool Profile::increasing( int i ) throw()
+bool Profile::increasing( int i, const int min_delta )
   {
-  if( _limit < 0 ) initialize();
-  if( i < 0 || i > samples() - 2 || data[samples()-1] - data[i] < 2 )
+  if( limit_ < 0 ) initialize();
+  if( i < 0 || i > samples() - 2 || data[samples()-1] - data[i] < min_delta )
     return false;
   while( ++i < samples() ) if( data[i] < data[i-1] ) return false;
   return true;
   }
 
 
-bool Profile::decreasing( int i ) throw()
+bool Profile::decreasing( int i, int end )
   {
-  if( _limit < 0 ) initialize();
-  const int noise = ( std::min( samples(), _limit ) / 20 ) + 1;
-  if( i < 0 || samples() - i <= 2 * noise ||
-      data[i] - data[samples()-noise] < noise + 1 )
+  if( limit_ < 0 ) initialize();
+  const int dnoise = ( samples() / 20 ) + 1;			// domain noise
+  const int rnoise = ( std::min( samples(), limit_ ) / 20 ) + 1; // range noise
+  if( end < 0 || end > samples() - dnoise ) end = samples() - dnoise;
+  if( i < 0 || end - i <= 2 * rnoise || data[i] - data[end-1] <= rnoise )
     return false;
-  while( ++i < samples() - noise ) if( data[i] > data[i-1] ) return false;
+  while( ++i < end ) if( data[i] > data[i-1] ) return false;
   return true;
   }
 
 
-bool Profile::isconcave() throw()
+bool Profile::isconcave()
   {
-  if( _isconcave < 0 )
+  if( isconcave_ < 0 )
     {
-    _isconcave = false; if( _limit < 0 ) initialize();
-    if( samples() < 5 ) return _isconcave;
+    isconcave_ = false; if( limit_ < 0 ) initialize();
+    if( samples() < 5 ) return isconcave_;
     int dmax = -1, l = 0, r = 0;
 
     for( int i = pos( 10 ); i <= pos( 90 ); ++i )
@@ -204,28 +205,28 @@ bool Profile::isconcave() throw()
       if( data[i] > dmax ) { dmax = data[i]; l = r = i; }
       else if( data[i] == dmax ) { r = i; }
       }
-    if( l > r || l < pos( 25 ) || r > pos( 75 ) ) return _isconcave;
-    if( data[pos(10)] >= dmax || data[pos(90)] >= dmax ) return _isconcave;
+    if( l > r || l < pos( 25 ) || r > pos( 75 ) ) return isconcave_;
+    if( data[pos(10)] >= dmax || data[pos(90)] >= dmax ) return isconcave_;
     int imax = ( l + r ) / 2;
 
     for( int i = pos( 10 ); i < imax; ++i )
-      if( data[i] > data[i+1] ) return _isconcave;
+      if( data[i] > data[i+1] ) return isconcave_;
     for( int i = pos( 90 ); i > imax; --i )
-      if( data[i] > data[i-1] ) return _isconcave;
-    _isconcave = true;
+      if( data[i] > data[i-1] ) return isconcave_;
+    isconcave_ = true;
     }
-  return _isconcave;
+  return isconcave_;
   }
 
 
-bool Profile::isconvex() throw()
+bool Profile::isconvex()
   {
-  if( _isconvex < 0 )
+  if( isconvex_ < 0 )
     {
-    _isconvex = false; if( _limit < 0 ) initialize();
-    if( samples() < 9 || _limit < 5 ) return _isconvex;
-    int min = _limit, min_begin = 0, min_end = 0;
-    int lmin = _limit, rmax = -_limit, l = 0, r = 0;
+    isconvex_ = false; if( limit_ < 0 ) initialize();
+    if( samples() < 9 || limit_ < 5 ) return isconvex_;
+    int min = limit_, min_begin = 0, min_end = 0;
+    int lmin = limit_, rmax = -limit_, l = 0, r = 0;
 
     for( int i = 1; i < samples(); ++i )
       {
@@ -235,118 +236,118 @@ bool Profile::isconvex() throw()
       if( data[i] <= min )
         { min_end = i; if( data[i] < min ) { min = data[i]; min_begin = i; } }
       }
-    if( l >= r || l >= pos( 25 ) || r <= pos( 75 ) ) return _isconvex;
+    if( l >= r || l >= pos( 25 ) || r <= pos( 75 ) ) return isconvex_;
     if( lmin >= 0 || rmax <= 0 || data[l] < 2 || data[r] < 2 ||
-        3 * ( data[l] + data[r] ) <= std::min( _limit, samples() ) )
-      return _isconvex;
-    if( 3 * ( min_end - min_begin + 1 ) > 2 * samples() ) return _isconvex;
-    if( 2 * l >= min_begin || 2 * r <= min_end + samples() - 1 ) return _isconvex;
-    if( min_begin < pos( 10 ) || min_end > pos( 90 ) ) return _isconvex;
+        3 * ( data[l] + data[r] ) <= std::min( samples(), limit_ ) )
+      return isconvex_;
+    if( 3 * ( min_end - min_begin + 1 ) > 2 * samples() ) return isconvex_;
+    if( 2 * l >= min_begin || 2 * r <= min_end + samples() - 1 ) return isconvex_;
+    if( min_begin < pos( 10 ) || min_end > pos( 90 ) ) return isconvex_;
 
-    const int noise = ( std::min( samples(), _limit ) / 30 ) + 1;
-    int dmax = -_limit;
+    const int noise = ( std::min( samples(), limit_ ) / 30 ) + 1;
+    int dmax = -limit_;
     for( int i = l + 1; i <= r; ++i )
       {
       if( i >= min_begin && i <= min_end )
-        { if( data[i] <= noise ) continue; else return _isconvex; }
+        { if( data[i] <= noise ) continue; else return isconvex_; }
       int d = data[i] - data[i-1];
       if( d == 0 ) continue;
       if( d > dmax ) { if( std::abs( d ) <= noise ) ++dmax; else dmax = d; }
-      else if( d < dmax - noise ) return _isconvex;
+      else if( d < dmax - noise ) return isconvex_;
       }
     if( 2 * ( min_end - min_begin + 1 ) < samples() )
       {
       int varea = ( min_begin - l + 1 ) * data[l] / 2;
       varea += ( r - min_end + 1 ) * data[r] / 2;
       if( this->area( l, min_begin - 1 ) + this->area( min_end + 1, r ) >= varea )
-        return _isconvex;
+        return isconvex_;
       }
-    _isconvex = true;
+    isconvex_ = true;
     }
-  return _isconvex;
+  return isconvex_;
   }
 
 
-bool Profile::isflat() throw()
+bool Profile::isflat()
   {
-  if( _isflat < 0 )
+  if( isflat_ < 0 )
     {
-    _isflat = false; if( _limit < 0 ) initialize();
-    if( samples() < 15 ) return _isflat;
+    isflat_ = false; if( limit_ < 0 ) initialize();
+    if( samples() < 10 ) return isflat_;
     int mn = data[samples()/2], mx = mn;
 
     for( int i = 1; i < samples() - 1; ++i )
       { int d = data[i]; if( d < mn ) mn = d; else if( d > mx ) mx = d; }
-    _isflat = (bool)( mx - mn <= 1 + ( samples() / 30 ) );
+    isflat_ = (bool)( mx - mn <= 1 + ( samples() / 30 ) );
     }
-  return _isflat;
+  return isflat_;
   }
 
 
-bool Profile::isflats() throw()
+bool Profile::isflats()
   {
-  if( _isflats < 0 )
+  if( isflats_ < 0 )
     {
-    _isflats = false; if( _limit < 0 ) initialize();
-    if( samples() < 15 ) return _isflats;
+    isflats_ = false; if( limit_ < 0 ) initialize();
+    if( samples() < 12 ) return isflats_;
     const int s1 = std::max( pos( 15 ), 3 );
     const int s2 = std::min( pos( 85 ), samples() - 4 );
     int mn = -1, mx = 0;
     for( int i = s1 + 2; i < s2; ++i )
       if( data[i-1] == data[i] ) { mn = mx = data[i]; break; }
-    if( mn < 0 ) return _isflats;
+    if( mn < 0 ) return isflats_;
 
     for( int i = 1; i <= s1; ++i ) if( data[i] > mx ) mx = data[i];
     for( int i = s1 + 1; i < s2; ++i )
       { int d = data[i]; if( d < mn ) mn = d; else if( d > mx ) mx = d; }
     for( int i = s2; i < samples() - 1; ++i ) if( data[i] > mx ) mx = data[i];
-    _isflats = (bool)( mx - mn <= 1 + ( samples() / 30 ) );
+    isflats_ = (bool)( mx - mn <= 1 + ( samples() / 30 ) );
     }
-  return _isflats;
+  return isflats_;
   }
 
 
-bool Profile::ispit() throw()
+bool Profile::ispit()
   {
-  if( _ispit < 0 )
+  if( ispit_ < 0 )
     {
-    _ispit = false; if( _limit < 0 ) initialize();
-    if( samples() < 5 ) return _ispit;
-    const int noise = ( std::min( samples(), _limit ) / 25 ) + 1;
+    ispit_ = false; if( limit_ < 0 ) initialize();
+    if( samples() < 5 ) return ispit_;
+    const int noise = ( std::min( samples(), limit_ ) / 25 ) + 1;
     for( int i = 0; i < noise; ++i )
       if( data[i] <= noise - i || data[samples()-i-1] <= noise - i )
-        return _ispit;
+        return ispit_;
 
-    const int dmin = min(), dmax = _limit / 2;
+    const int dmin = min(), dmax = limit_ / 2;
     int begin = 0, end = 0, i, ref;
     for( i = 0, ref = dmax; i < samples(); ++i )
       {
       int d = data[i];
       if( d == dmin ) { begin = i; break; }
-      if( d < ref ) ref = d; else if( d > ref + noise && ref < dmax ) return _ispit;
+      if( d < ref ) ref = d; else if( d > ref + noise && ref < dmax ) return ispit_;
       }
-    if( begin < 2 || begin > samples() - 3 ) return _ispit;
+    if( begin < 2 || begin > samples() - 3 ) return ispit_;
 
     for( i = samples() - 1, ref = dmax; i >= begin; --i )
       {
       int d = data[i];
       if( d == dmin ) { end = i; break; }
-      if( d < ref ) ref = d; else if( d > ref + noise && ref < dmax ) return _ispit;
+      if( d < ref ) ref = d; else if( d > ref + noise && ref < dmax ) return ispit_;
       }
-    if( end < begin || end > samples() - 3 ) return _ispit;
+    if( end < begin || end > samples() - 3 ) return ispit_;
 
     for( i = begin + 1; i < end; ++i )
-      if( data[i] > dmin + noise ) return _ispit;
+      if( data[i] > dmin + noise ) return ispit_;
 
-    _ispit = true;
+    ispit_ = true;
     }
-  return _ispit;
+  return ispit_;
   }
 
 
-bool Profile::iscpit( const int cpos ) throw()
+bool Profile::iscpit( const int cpos )
   {
-  if( _limit < 0 ) initialize();
+  if( limit_ < 0 ) initialize();
   if( samples() < 5 || cpos < 25 || cpos > 75 ) return false;
   const int mid = ( ( samples() - 1 ) * cpos ) / 100;
   const int iend = std::min( samples() / 4, std::min( mid, samples() - mid ) );
@@ -370,50 +371,30 @@ bool Profile::iscpit( const int cpos ) throw()
   }
 
 
-bool Profile::islpit() throw()
+bool Profile::istpit()
   {
-  if( _limit < 0 ) initialize();
-  if( samples() < 5 ) return false;
-  const int noise = samples() / 30;
-  if( data[0] < noise + 2 ) return false;
-
-  const int dmin = min();
-  int begin = 0, ref = _limit;
-  for( int i = 0; i < samples(); ++i )
+  if( istpit_ < 0 )
     {
-    int d = data[i];
-    if( d == dmin ) { begin = i; break; }
-    if( d < ref ) ref = d; else if( d > ref + 1 ) return false;
-    }
-  if( begin < 2 || 2 * begin >= samples() ) return false;
-  return true;
-  }
+    if( limit_ < 0 ) initialize();
+    if( limit_ < 5 || samples() < 5 || !ispit() )
+      { istpit_ = false; return istpit_; }
 
-
-bool Profile::istpit() throw()
-  {
-  if( _istpit < 0 )
-    {
-    if( _limit < 0 ) initialize();
-    if( _limit < 5 || samples() < 5 || !ispit() )
-      { _istpit = false; return _istpit; }
-
-    const int noise = ( std::min( _limit, samples() ) / 30 ) + 1;
+    const int noise = ( std::min( samples(), limit_ ) / 30 ) + 1;
     int l = -1, r = 0;
     for( int i = 0; i < samples(); ++i )
       if( data[i] <= noise ) { r = i; if( l < 0 ) l = i; }
-    _istpit = (bool)( l > 0 && 4 * ( r - l + 1 ) < samples() );
+    istpit_ = (bool)( l > 0 && 4 * ( r - l + 1 ) < samples() );
     }
-  return _istpit;
+  return istpit_;
   }
 
 
-bool Profile::isupit() throw()
+bool Profile::isupit()
   {
-  if( _isupit < 0 )
+  if( isupit_ < 0 )
     {
-    _isupit = false; if( _limit < 0 ) initialize();
-    if( samples() < 5 ) return _isupit;
+    isupit_ = false; if( limit_ < 0 ) initialize();
+    if( samples() < 5 ) return isupit_;
 
     int th = ( mean() < 2 && range() > 2 ) ? 2 : mean();
     int status = 0, ucount = 0, lcount = 0, umean =0, lmean = 0;
@@ -423,38 +404,38 @@ bool Profile::isupit() throw()
       switch( status )
         {
         case 0: if( d < th )
-                  { if( i < pos( 25 ) || i > pos( 70 ) ) return _isupit;
+                  { if( i < pos( 25 ) || i > pos( 70 ) ) return isupit_;
                   status = 1; break; }
                 if( d > th ) { ++ucount; umean += d; }
           break;
         case 1: if( d > th )
-                  { if( i < pos( 30 ) || i > pos( 75 ) ) return _isupit;
+                  { if( i < pos( 30 ) || i > pos( 75 ) ) return isupit_;
                   status = 2; break; }
                 if( d < th ) { ++lcount; lmean += d; }
           break;
-        case 2: if( d < th ) return _isupit;
+        case 2: if( d < th ) return isupit_;
                 if( d > th ) { ++ucount; umean += d; }
           break;
         }
       }
     if( ucount > 1 ) umean /= ucount;
     if( lcount > 1 ) lmean /= lcount;
-    _isupit = (bool)( status == 2 && umean - lmean > range() / 2 );
+    isupit_ = (bool)( status == 2 && umean - lmean > range() / 2 );
     }
-  return _isupit;
+  return isupit_;
   }
 
 
-bool Profile::isvpit() throw()
+bool Profile::isvpit()
   {
-  if( _isvpit < 0 )
+  if( isvpit_ < 0 )
     {
-    if( _limit < 0 ) initialize();
-    if( _limit < 5 || samples() < 5 || !ispit() )
-      { _isvpit = false; return _isvpit; }
+    if( limit_ < 0 ) initialize();
+    if( limit_ < 5 || samples() < 5 || !ispit() )
+      { isvpit_ = false; return isvpit_; }
 
-    const int noise = _limit / 20;
-    const int level = ( _limit / 10 ) + 2;
+    const int noise = limit_ / 20;
+    const int level = ( limit_ / 10 ) + 2;
     int ll = -1, ln = -1, rl = -1, rn = -1;
     for( int i = 0; i < samples(); ++i )
       if( data[i] <= level )
@@ -463,30 +444,31 @@ bool Profile::isvpit() throw()
         if( data[i] <= noise ) { rn = i; if( ln < 0 ) ln = i; }
         }
     const int wl = rl - ll + 1, wn = rn - ln + 1;
-    _isvpit = (bool)( ln > 0 && 2 * wl <= samples() + 1 && wl - wn <= 2 * ( level - noise ) );
+    isvpit_ = (bool)( ln > 0 && 2 * wl <= samples() + 1 && wl - wn <= 2 * ( level - noise ) );
     }
-  return _isvpit;
+  return isvpit_;
   }
 
 
-bool Profile::istip() throw()
+bool Profile::istip()
   {
-  if( _istip < 0 )
+  if( istip_ < 0 )
     {
-    _istip = false; if( _limit < 0 ) initialize();
-    if( samples() < 5 ) return _istip;
+    istip_ = false; if( limit_ < 0 ) initialize();
+    if( samples() < 5 ) return istip_;
 
     int th = ( mean() < 2 && range() > 2 ) ? 2 : mean(); if( th < 2 ) ++th;
     int lth = data[0], rth = data[samples()-1];
     int begin = 0, end = samples() - 1;
-    for( int i = 1, j = std::max( 2, samples() / 10 ); i < j; ++i )
+    const int tries = std::max( 2, ( samples() + 5 ) / 10 );
+    for( int i = 1; i < tries; ++i )
       {
       if( data[i] < lth ) { lth = data[i]; begin = i; }
       if( data[samples()-1-i] < rth )
         { rth = data[samples()-1-i]; end = samples() - 1 - i; }
       }
-    if( lth >= th || rth >= th ) return _istip;
-    if( 3 * lth >= 2 * range() || 3 * rth >= 2 * range() ) return _istip;
+    if( lth >= th || rth >= th ) return istip_;
+    if( 3 * lth >= 2 * range() || 3 * rth >= 2 * range() ) return istip_;
     th = std::max( lth, rth );
     int status = 0;
     for( int i = begin + 1; i < end; ++i )
@@ -495,21 +477,21 @@ bool Profile::istip() throw()
         case 0: if( data[i] > th + 1 ) status = 1; break;
         case 1: if( data[i] > th + 1 ) status = 2; else status = 0; break;
         case 2: if( data[i] <= th ) status = 3; break;
-        case 3: if( data[i] > th + 1 ) return _istip;
+        case 3: if( data[i] > th + 1 ) return istip_;
         }
-    _istip = (bool)( status >= 2 );
+    istip_ = (bool)( status >= 2 );
     }
-  return _istip;
+  return istip_;
   }
 
 
-bool Profile::isctip( const int cpos ) throw()
+bool Profile::isctip( const int cpos )
   {
-  if( _limit < 0 ) initialize();
+  if( limit_ < 0 ) initialize();
   if( samples() < 5 || cpos < 25 || cpos > 75 ) return false;
   const int mid = ( ( samples() - 1 ) * cpos ) / 100;
   const int iend = std::min( samples() / 4, std::min( mid, samples() - mid ) );
-  int th = std::max( 2, std::min( mean(), _limit / 3 ) );
+  int th = std::max( 2, std::min( mean(), limit_ / 3 ) );
   int imax = -1;
 
   for( int i = 0; i < iend; ++i )
@@ -539,9 +521,49 @@ bool Profile::isctip( const int cpos ) throw()
   }
 
 
-int Profile::imaximum() throw()
+bool Profile::isltip()
   {
-  if( _limit < 0 ) initialize();
+  if( limit_ < 0 ) initialize();
+  if( samples() < 5 ) return false;
+  const int noise = ( samples() / 30 ) + 1;
+  if( data[0] <= noise ) return false;
+
+  const int dmin = min();
+  int begin = 0, ref = limit_;
+  for( int i = 0; i < samples() - noise; ++i )
+    {
+    int d = data[i];
+    if( d == dmin ) { begin = i; break; }
+    if( d < ref ) ref = d; else if( d > ref + noise ) return false;
+    }
+  if( begin <= noise || 2 * begin > samples() ) return false;
+  return true;
+  }
+
+
+bool Profile::isrtip()
+  {
+  if( limit_ < 0 ) initialize();
+  if( samples() < 5 ) return false;
+  const int noise = ( samples() / 30 ) + 1;
+  if( data[samples()-1] <= noise ) return false;
+
+  const int dmin = min();
+  int begin = 0, ref = limit_;
+  for( int i = samples() - 1; i >= noise; --i )
+    {
+    int d = data[i];
+    if( d == dmin ) { begin = samples() - 1 - i; break; }
+    if( d < ref ) ref = d; else if( d > ref + noise ) return false;
+    }
+  if( begin <= noise || 2 * begin > samples() ) return false;
+  return true;
+  }
+
+
+int Profile::imaximum()
+  {
+  if( limit_ < 0 ) initialize();
   const int margin = ( samples() / 30 ) + 1;
   int mbegin = 0, mend, mvalue = 0;
 
@@ -555,14 +577,14 @@ int Profile::imaximum() throw()
   }
 
 
-int Profile::iminimum( int m, int th ) throw()
+int Profile::iminimum( const int m, int th )
   {
-  if( _limit < 0 ) initialize();
+  if( limit_ < 0 ) initialize();
   const int margin = ( samples() / 30 ) + 1;
   if( samples() < 2 * margin ) return 0;
   if( th < 2 ) th = ( mean() < 2 ) ? 2 : mean();
   int minima = 0, status = 0;
-  int begin = 0, end, value = _limit + 1;
+  int begin = 0, end, value = limit_ + 1;
 
   for( end = margin; end < samples() - margin; ++end )
     {
@@ -581,12 +603,12 @@ int Profile::iminimum( int m, int th ) throw()
   }
 
 
-int Profile::minima( int th ) throw()
+int Profile::minima( int th )
   {
-  if( _limit < 0 ) initialize();
+  if( limit_ < 0 ) initialize();
   if( !samples() ) return 0;
   if( th < 1 ) th = ( mean() < 2 ) ? 2 : mean();
-  const int noise = _limit / 40;
+  const int noise = limit_ / 40;
   const int dth = th - ( ( noise + 1 ) / 2 ), uth = th + ( noise / 2 );
   if( dth < 1 ) return 1;
   int minima = ( data[0] < dth ) ? 1 : 0;
@@ -602,9 +624,9 @@ int Profile::minima( int th ) throw()
   }
 
 
-bool Profile::straight( int * _dy ) throw()
+bool Profile::straight( int * const dyp )
   {
-  if( _limit < 0 ) initialize();
+  if( limit_ < 0 ) initialize();
   if( samples() < 5 ) return false;
 
   const int xl = ( samples() / 30 ) + 1, yl = ( data[xl] + data[xl+1] ) / 2 ;
@@ -620,6 +642,6 @@ bool Profile::straight( int * _dy ) throw()
     if( d >= dmax && ( ( dx * data[i] ) < y || ( i >= xl && i <= xr ) ) )
       if( d > dmax || ( d == dmax && --faults < 0 ) ) return false;
     }
-  if( _dy ) *_dy = dy;
+  if( dyp ) *dyp = dy;
   return true;
   }
