@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2003-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -16,13 +16,18 @@
 */
 
 #include <algorithm>
+#include <climits>
 #include <cstdio>
+#include <string>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
 #include "rational.h"
 #include "rectangle.h"
 #include "track.h"
+#include "ucs.h"
+#include "user_filter.h"
 #include "bitmap.h"
 #include "blob.h"
 #include "character.h"
@@ -33,15 +38,11 @@
 
 namespace {
 
-void add_line( std::vector< Textline * > & textlinep_vector ) throw()
-  { textlinep_vector.push_back( new Textline() ); }
+void insert_line( std::vector< Textline * > & textlinep_vector, int i )
+  { textlinep_vector.insert( textlinep_vector.begin() + i, new Textline ); }
 
 
-void insert_line( std::vector< Textline * > & textlinep_vector, int i ) throw()
-  { textlinep_vector.insert( textlinep_vector.begin() + i, new Textline() ); }
-
-
-void delete_line( std::vector< Textline * > & textlinep_vector, int i ) throw()
+void delete_line( std::vector< Textline * > & textlinep_vector, int i )
   {
   delete textlinep_vector[i];
   textlinep_vector.erase( textlinep_vector.begin() + i );
@@ -50,9 +51,9 @@ void delete_line( std::vector< Textline * > & textlinep_vector, int i ) throw()
 
 // Build the vertical composite characters.
 //
-void join_characters( std::vector< Textline * > & tlpv ) throw()
+void join_characters( std::vector< Textline * > & tlpv )
   {
-  for( unsigned int current_line = 0; current_line < tlpv.size(); ++current_line )
+  for( unsigned current_line = 0; current_line < tlpv.size(); ++current_line )
     {
     Textline & line = *tlpv[current_line];
     for( int i = 0 ; i < line.characters() - 1; )
@@ -89,105 +90,169 @@ void join_characters( std::vector< Textline * > & tlpv ) throw()
 } // end namespace
 
 
-Textblock::Textblock( const Rectangle & r,
-                      std::vector< std::vector< Blob * > > & blobp_matrix ) throw()
-  : Rectangle( r )
+void Textblock::apply_filters( const Control & control )
   {
-  int cuts = blobp_matrix.size();
-  std::vector< int > mean_height( cuts, 0 );
-  std::vector< std::vector< Blob * > > pending( cuts );
-  std::vector< std::vector< Blob * > > pending_tall( cuts );
-  std::vector< std::vector< Blob * > > pending_short( cuts );
-
-  // Classify blobs by height.
-  for( int cut = 0; cut < cuts; ++cut )
+  if( textlines() <= 0 ) return;
+  for( unsigned f = 0; f < control.filters.size(); ++f )
     {
-    const std::vector< Blob * > & blobp_vector = blobp_matrix[cut];
-    if( !blobp_vector.size() ) continue;
-    unsigned int samples = 0;
-    std::vector< int > height_distrib;
-    for( unsigned int i = 0; i < blobp_vector.size(); ++i )
+    if( control.filters[f].user_filterp )
       {
-      unsigned int h = blobp_vector[i]->height();
-      unsigned int w = blobp_vector[i]->width();
-      if( h < 10 || w >= 3 * h ) continue;
+      for( int i = 0; i < textlines(); ++i )
+        tlpv[i]->apply_user_filter( *control.filters[f].user_filterp );
+      continue;
+      }
+    const Filter::Type filter = control.filters[f].type;
+    if( filter != Filter::text_block )
+      {
+      for( int i = 0; i < textlines(); ++i )
+        tlpv[i]->apply_filter( filter );
+      continue;
+      }
+    int l = right(), t = bottom(), r = left(), b = top();
+    for( int i = 0; i < textlines(); ++i )
+      {
+      const Textline & line = *tlpv[i];
+      int first = line.characters(), last = -1;
+      for( int j = line.big_initials(); j < line.characters(); ++j )
+        if( line.is_key_character( j ) )
+          { l = std::min( l, line.character(j).left() ); first = j; break; }
+      for( int j = line.characters() - 1; j >= first; --j )
+        if( line.is_key_character( j ) )
+          { r = std::max( r, line.character(j).right() ); last = j; break; }
+      if( i == 0 )
+        { for( int j = first; j <= last; ++j )
+            if( line.is_key_character( j ) )
+              t = std::min( t, line.character(j).top() ); }
+      else if( i == textlines() - 1 )
+        { for( int j = first; j <= last; ++j )
+            if( line.is_key_character( j ) )
+              b = std::max( b, line.character(j).bottom() ); }
+      }
+    if( r < l || b < t ) continue;		// can't apply filter; no text
+    Rectangle re( l, t, r, b );
+    for( int i = 0; i < textlines(); ++i )
+      {
+      Textline & line = *tlpv[i];
+      bool modified = false;
+      for( int j = line.characters() - 1; j >= 0; --j )
+        if( line.character(j).height() >= 2 * line.height() ||
+            !re.includes( line.character(j).vcenter(), line.character(j).hcenter() ) )
+          { line.delete_character( j ); modified = true; }
+      if( modified ) line.remove_leadind_trailing_duplicate_spaces();
+      }
+    }
+  }
+
+
+Textblock::Textblock( const Rectangle & page, const Rectangle & block,
+                      std::vector< Blob * > & blobp_vector )
+  : Rectangle( block )
+  {
+  std::vector< Blob * > pending;
+  std::vector< Blob * > pending_tall;
+  std::vector< Blob * > pending_short;
+
+  for( unsigned begin = 0, end = 0; end < blobp_vector.size(); begin = end )
+    {
+    int botmax = blobp_vector[begin]->bottom();		// make cuts
+    while( ++end < blobp_vector.size() )
+      {
+      if( blobp_vector[end]->top() > botmax ) break;
+      botmax = std::max( botmax, blobp_vector[end]->bottom() );
+      }
+
+    // Classify blobs by height.
+    unsigned samples = 0;
+    std::vector< int > height_distrib;
+    for( unsigned i = begin; i < end; ++i )
+      {
+      if( blobp_vector[i]->is_abnormal() ) continue;
+      unsigned h = blobp_vector[i]->height();
       if( h >= height_distrib.size() ) height_distrib.resize( h + 1 );
       ++height_distrib[h]; ++samples;
       }
-    if( !height_distrib.size() )
-      for( unsigned int i = 0; i < blobp_vector.size(); ++i )
+    if( height_distrib.empty() )		// all blobs are abnormal
+      for( unsigned i = begin; i < end; ++i )
         {
-        unsigned int h = blobp_vector[i]->height();
+        unsigned h = blobp_vector[i]->height();
         if( h >= height_distrib.size() ) height_distrib.resize( h + 1 );
         ++height_distrib[h]; ++samples;
         }
 
+    int mean_height = 0;
     int valid_samples = 0;
-    for( unsigned int i = 0, count = 0; i < height_distrib.size(); ++i )
+    for( unsigned i = 0, count = 0; i < height_distrib.size(); ++i )
       {
       int a = height_distrib[i];
       if( 10 * ( count + a ) >= samples && 10 * count < 9 * samples )
-        { mean_height[cut] += a * i; valid_samples += a; }
+        { mean_height += a * i; valid_samples += a; }
       count += a;
       }
-    if( valid_samples ) mean_height[cut] /= valid_samples;
+    if( valid_samples ) mean_height /= valid_samples;
 
-    for( unsigned int i = 0; i < blobp_vector.size(); ++i )
+    for( unsigned i = begin; i < end; ++i )
       {
-      Blob * p = blobp_vector[i];
-      if( p->height() >= 2 * mean_height[cut] )
-        pending_tall[cut].push_back( p );
-      else if( 2 * p->height() <= mean_height[cut] )
-        pending_short[cut].push_back( p );
-      else pending[cut].push_back( p );
+      Blob * const p = blobp_vector[i];
+      const bool a = p->is_abnormal();
+      if( p->height() >= 2 * mean_height ||
+          ( a && p->height() > mean_height ) ) pending_tall.push_back( p );
+      else if( 2 * p->height() <= mean_height || p->height() <= 5 ||
+               ( a && p->height() < mean_height ) )
+        pending_short.push_back( p );
+      else pending.push_back( p );
       }
     }
 
-  // Assign normal blobs to characters and create lines.
-  int min_line = 0;	// first line of current cut
-  add_line( tlpv );
-  for( int cut = 0; cut < cuts; ++cut )
+  if( pending.empty() )
     {
-    if( pending[cut].size() )
+    for( unsigned i = 0; i < blobp_vector.size(); ++i )
+      delete blobp_vector[i];
+    blobp_vector.clear();
+    return;
+    }
+  blobp_vector.clear();
+
+  // Assign normal blobs to characters and create lines.
+  {
+  int min_line = 0;			// first line of current cut
+  tlpv.push_back( new Textline );
+  int current_line = min_line = textlines() - 1;
+  tlpv[current_line]->shift_characterp( new Character( pending[0] ) );
+  for( unsigned i = 1; i < pending.size(); ++i )
+    {
+    Blob & b = *pending[i];
+    current_line = std::max( min_line, current_line - 2 );
+    while( true )
       {
-      if( tlpv.back()->characters() ) add_line( tlpv );
-      int current_line = min_line = textlines() - 1;
-      tlpv[current_line]->shift_characterp( new Character( pending[cut][0] ) );
-      for( unsigned int i = 1; i < pending[cut].size(); ++i )
+      const Character *cl = 0, *cr = 0;
+      for( int j = tlpv[current_line]->characters() - 1; j >= 0; --j )
         {
-        Blob & b = *pending[cut][i];
-        current_line = std::max( min_line, current_line - 2 );
-        while( true )
-          {
-          const Character *cl = 0, *cr = 0;
-          for( int j = tlpv[current_line]->characters() - 1; j >= 0; --j )
-            {
-            const Character & cj = tlpv[current_line]->character( j );
-            if( !b.includes_hcenter( cj ) && !cj.includes_hcenter( b ) )
-              { if( b.h_precedes( cj ) ) cr = &cj; else { cl = &cj; break; } }
-            }
-          if( ( cl && ( cl->includes_vcenter( b ) || b.includes_vcenter( *cl ) ) ) ||
-              ( cr && ( cr->includes_vcenter( b ) || b.includes_vcenter( *cr ) ) ) )
-            { tlpv[current_line]->shift_characterp( new Character( &b ) ); break; }
-          else if( ( cl && cl->bottom() < b.top() ) || ( cr && cr->bottom() < b.top() ) )
-            {
-            if( ++current_line >= textlines() )
-              { add_line( tlpv ); current_line = textlines() - 1;
-              tlpv[current_line]->shift_characterp( new Character( &b ) ); break; }
-            }
-          else if( ( cl && cl->top() > b.bottom() ) || ( cr && cr->top() > b.bottom() ) )
-            {
-            insert_line( tlpv, current_line );
-            tlpv[current_line]->shift_characterp( new Character( &b ) ); break;
-            }
-          else if( ( cl && cl->v_overlaps( b ) ) || ( cr && cr->v_overlaps( b ) ) )
-            { tlpv[current_line]->shift_characterp( new Character( &b ) ); break; }
-          else { delete &b; break; }
-          }
+        const Character & cj = tlpv[current_line]->character( j );
+        if( !b.includes_hcenter( cj ) && !cj.includes_hcenter( b ) )
+          { if( b.h_precedes( cj ) ) cr = &cj; else { cl = &cj; break; } }
+        }
+      if( ( cl && ( cl->includes_vcenter( b ) || b.includes_vcenter( *cl ) ) ) ||
+          ( cr && ( cr->includes_vcenter( b ) || b.includes_vcenter( *cr ) ) ) )
+        { tlpv[current_line]->shift_characterp( new Character( &b ) ); break; }
+      else if( ( cl && cl->top() > b.bottom() ) || ( cr && cr->top() > b.bottom() ) )
+        {
+        insert_line( tlpv, current_line );
+        tlpv[current_line]->shift_characterp( new Character( &b ) ); break;
+        }
+      else if( ( cl && cl->v_overlap_percent( b ) > 5 ) ||
+               ( cr && cr->v_overlap_percent( b ) > 5 ) )
+        { tlpv[current_line]->shift_characterp( new Character( &b ) ); break; }
+      else if( ++current_line >= textlines() )
+        {
+        tlpv.push_back( new Textline ); current_line = textlines() - 1;
+        tlpv[current_line]->shift_characterp( new Character( &b ) ); break;
         }
       }
     }
 
+  for( int i = textlines() - 1; i >= 0; --i )
+    if( !tlpv[i]->characters() ) delete_line( tlpv, i );
+
   join_characters( tlpv );
 
   // Create tracks of lines.
@@ -195,80 +260,78 @@ Textblock::Textblock( const Rectangle & r,
 
   // Insert tall blobs.
   // Seek up, then seek down, needed for slanted or curved lines.
-  for( int current_line = 0, cut = 0; cut < cuts; ++cut )
+  current_line = 0;
+  for( unsigned i = 0; i < pending_tall.size(); ++i )
     {
-    for( unsigned int i = 0; i < pending_tall[cut].size(); ++i )
-      {
-      Blob & b = *pending_tall[cut][i];
-      while( current_line > 0 &&
-             b.bottom() < tlpv[current_line]->vcenter( b.hcenter() ) )
-        --current_line;
-      while( current_line < textlines() &&
-             b.top() > tlpv[current_line]->vcenter( b.hcenter() ) )
-        ++current_line;
-      if( current_line >= textlines() )
-        { --current_line; delete &b; continue; }
-      Textline & l = *tlpv[current_line];
-      const int bi = l.big_initials();
-      const int mh = l.mean_height();
-      if( b.height() <= 3 * mh &&
-          ( b.height() <= 2 * mh || l.character( bi ).left() < b.left() ) )
-        l.shift_characterp( new Character( &b ) );
-      else		// possible big initial
-        {
-        if( l.character( std::min( bi+1, l.characters() - 1 ) ).left() > b.hcenter() )
-          l.shift_characterp( new Character( &b ), true );
-        }
-      }
+    Blob & b = *pending_tall[i];
+    while( current_line > 0 &&
+           b.bottom() < tlpv[current_line]->vcenter( b.hcenter() ) )
+      --current_line;
+    while( current_line < textlines() &&
+           b.top() > tlpv[current_line]->vcenter( b.hcenter() ) )
+      ++current_line;
+    if( current_line >= textlines() )
+      { --current_line; delete &b; continue; }
+    Textline & l = *tlpv[current_line];
+    const int bi = l.big_initials();
+    const int mh = l.mean_height();
+    if( b.height() <= 3 * mh &&
+        ( b.height() <= 2 * mh || l.character( bi ).left() < b.left() ) )
+      l.shift_characterp( new Character( &b ) );
+    else if( !l.characters() ||
+             l.character( std::min( bi+1, l.characters() - 1 ) ).left() > b.hcenter() )
+      l.shift_characterp( new Character( &b ), true );	// big initial
+    else delete &b;
     }
 
 //  for( int i = 0; i < textlines(); ++i ) tlpv[i]->verify_big_initials();
 
   // Insert short blobs.
   // Seek up, then seek down, needed for slanted or curved lines.
-  for( int current_line = 0, cut = 0; cut < cuts; ++cut )
+  current_line = 0;
+  for( unsigned i = 0; i < pending_short.size(); ++i )
     {
-    for( unsigned int i = 0; i < pending_short[cut].size(); ++i )
+    Blob & b = *pending_short[i];
+    while( current_line > 0 &&
+           b.bottom() < tlpv[current_line]->top( b.hcenter() ) )
+      --current_line;
+    int temp = std::max( 0, current_line - 1 );
+    while( current_line < textlines() &&
+           b.top() > tlpv[current_line]->bottom( b.hcenter() ) )
+      ++current_line;
+    if( current_line >= textlines() )
       {
-      Blob & b = *pending_short[cut][i];
-      while( current_line > 0 &&
-             b.bottom() < tlpv[current_line]->top( b.hcenter() ) )
-        --current_line;
-      int temp = std::max( 0, current_line - 1 );
-      while( current_line < textlines() &&
-             b.top() > tlpv[current_line]->bottom( b.hcenter() ) )
-        ++current_line;
-      if( current_line >= textlines() )
-        {
-        const Textline & l = *tlpv[--current_line];
-        if( b.top() > l.bottom( b.hcenter() ) + l.height() ) continue;
-        else temp = current_line;
-        }
-      if( current_line - temp > 1 ) temp = current_line - 1;
-      if( current_line != temp &&
-          2 * ( b.top() - tlpv[temp]->bottom( b.hcenter() ) ) <
-          tlpv[current_line]->top( b.hcenter() ) - b.bottom() )
-        current_line = temp;
-      tlpv[current_line]->shift_characterp( new Character( &b ) );
+      const Textline & l = *tlpv[--current_line];
+      const Character *p = l.character_at( b.hcenter() );
+      if( b.top() > l.bottom( b.hcenter() ) + l.height() / 2 &&
+          ( !p || b.top() > p->bottom() + l.height() / 2 ) )
+        { delete &b; continue; }
+      else temp = current_line;
       }
+    if( current_line - temp > 1 ) temp = current_line - 1;
+    if( current_line != temp &&
+        2 * ( b.top() - tlpv[temp]->bottom( b.hcenter() ) ) <
+        tlpv[current_line]->top( b.hcenter() ) - b.bottom() )
+      current_line = temp;
+    tlpv[current_line]->shift_characterp( new Character( &b ) );
     }
 
-  // remove clipped lines at top or bottom of text block
-  if( textlines() )
+  // remove clipped lines at top or bottom of page
+  if( textlines() > 2 )
     {
-    const Textline & l = *tlpv[textlines()-1];
-    for( int i = 0, c = 0; i < l.characters(); ++i )
-      if( l.character( i ).bottom() >= bottom() && 2 * ++c >= l.characters() )
+    const Textline * lp = tlpv[textlines()-1];
+    for( int i = 0, c = 0; i < lp->characters(); ++i )
+      if( lp->character( i ).bottom() >= page.bottom() &&
+          2 * ++c >= lp->characters() )
         { delete_line( tlpv, textlines() - 1 ); break; }
-    }
-  if( textlines() )
-    {
-    const Textline & l = *tlpv[0];
-    const int t = std::max( top(), 1 );
-    for( int i = 0, c = 0; i < l.characters(); ++i )
-      if( l.character( i ).top() <= t && 2 * ++c >= l.characters() )
+
+    lp = tlpv[0];
+    const int t = std::max( page.top(), 1 );
+    for( int i = 0, c = 0; i < lp->characters(); ++i )
+      if( lp->character( i ).top() <= t && 2 * ++c >= lp->characters() )
         { delete_line( tlpv, 0 ); break; }
     }
+  }
 
   // Second pass. Join lines of i-dots and tildes.
   for( int current_line = 0; current_line < textlines() - 1; )
@@ -347,35 +410,33 @@ Textblock::Textblock( const Rectangle & r,
   }
 
 
-Textblock::~Textblock() throw()
+Textblock::~Textblock()
   {
   for( int i = textlines() - 1; i >= 0; --i ) delete tlpv[i];
   }
 
 
-void Textblock::recognize( const Charset & charset, const Filter & filter ) throw()
+void Textblock::recognize( const Control & control )
   {
   // Recognize characters.
   for( int i = 0; i < textlines(); ++i )
     {
     // First pass. Recognize the easy characters.
-    tlpv[i]->recognize1( charset );
+    tlpv[i]->recognize1( control.charset );
     // Second pass. Use context to clear up ambiguities.
-    tlpv[i]->recognize2( charset );
+    tlpv[i]->recognize2( control.charset );
     }
 
-  if( filter.type() != Filter::none )
-    for( int i = 0; i < textlines(); ++i )
-      tlpv[i]->apply_filter( filter );
+  apply_filters( control );
 
   // Remove unrecognized lines.
   for( int i = textlines() - 1; i >= 0; --i )
     {
     Textline & line1 = *tlpv[i];
-    bool flag = false;
+    bool recognized = false;
     for( int j = 0 ; j < line1.characters(); ++j )
-      { if( line1.character( j ).guesses() ) { flag = true; break; } }
-    if( !flag ) delete_line( tlpv, i );
+      { if( line1.character( j ).guesses() ) { recognized = true; break; } }
+    if( !recognized ) delete_line( tlpv, i );
     }
 
   // Add blank lines.
@@ -408,7 +469,15 @@ void Textblock::recognize( const Charset & charset, const Filter & filter ) thro
   }
 
 
-int Textblock::characters() const throw()
+const Textline & Textblock::textline( const int i ) const
+  {
+  if( i < 0 || i >= textlines() )
+    Ocrad::internal_error( "line, index out of bounds." );
+  return *tlpv[i];
+  }
+
+
+int Textblock::characters() const
   {
   int total = 0;
   for( int i = 0; i < textlines(); ++i )
@@ -416,16 +485,8 @@ int Textblock::characters() const throw()
   return total;
   }
 
-/*
-Textline & Textblock::textline( int i ) const throw()
-  {
-  if( i < 0 || i >= textlines() )
-    Ocrad::internal_error( "line, index out of bounds" );
-  return *tlpv[i];
-  }
-*/
 
-void Textblock::print( const Control & control ) const throw()
+void Textblock::print( const Control & control ) const
   {
   for( int i = 0; i < textlines(); ++i )
     tlpv[i]->print( control );
@@ -433,8 +494,7 @@ void Textblock::print( const Control & control ) const throw()
   }
 
 
-void Textblock::dprint( const Control & control, bool graph, bool recursive )
-								const throw()
+void Textblock::dprint( const Control & control, bool graph, bool recursive ) const
   {
   std::fprintf( control.outfile, "%d lines\n\n", textlines() );
 
@@ -448,7 +508,7 @@ void Textblock::dprint( const Control & control, bool graph, bool recursive )
   }
 
 
-void Textblock::xprint( const Control & control ) const throw()
+void Textblock::xprint( const Control & control ) const
   {
   std::fprintf( control.exportfile, "lines %d\n", textlines() );
 
@@ -461,13 +521,13 @@ void Textblock::xprint( const Control & control ) const throw()
   }
 
 
-void Textblock::cmark( Page_image & page_image ) const throw()
+void Textblock::cmark( Page_image & page_image ) const
   {
   for( int i = 0; i < textlines(); ++i ) tlpv[i]->cmark( page_image );
   }
 
 
-void Textblock::lmark( Page_image & page_image ) const throw()
+void Textblock::lmark( Page_image & page_image ) const
   {
   for( int i = 0; i < textlines(); ++i ) page_image.draw_track( *tlpv[i] );
   }
